//+--------------------------------------------------------------------+
//| Copyright:  (C) 2014, Miroslav Popov - All rights reserved!        |
//| Website:    http://forexsb.com/                                    |
//| Support:    http://forexsb.com/forum/                              |
//| License:    Proprietary under the following circumstances:         |
//|                                                                    |
//| This code is a part of Forex Strategy Builder. It is free for      |
//| use as an integral part of Forex Strategy Builder.                 |
//| One can modify it in order to improve the code or to fit it for    |
//| personal use. This code or any part of it cannot be used in        |
//| another applications without a permission. Contact information     |
//| cannot be changed.                                                 |
//|                                                                    |
//| NO LIABILITY FOR CONSEQUENTIAL DAMAGES                             |
//|                                                                    |
//| In no event shall the author be liable for any damages whatsoever  |
//| (including, without limitation, incidental, direct, indirect and   |
//| consequential damages, damages for loss of business profits,       |
//| business interruption, loss of business information, or other      |
//| pecuniary loss) arising out of the use or inability to use this    |
//| product, even if advised of the possibility of such damages.       |
//+--------------------------------------------------------------------+

#property copyright "Copyright 2014, Miroslav Popov"
#property link      "http://forexsb.com"
#property version   "1.00"
#property strict

#include <Forexsb.com/Indicator.mqh>
#include <Forexsb.com/Enumerations.mqh>
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class LSMA_Channelv2 : public Indicator
  {
public:
                     LSMA_Channelv2(SlotTypes slotType)
     {
      SlotType=slotType;

      IndicatorName="LSMA_Channel v2";

      WarningMessage    = "";
      IsAllowLTF        = true;
      ExecTime          = ExecutionTime_DuringTheBar;
      IsSeparateChart   = false;
      IsDiscreteValues  = false;
      IsDeafultGroupAll = false;
     }

   virtual void      Calculate(DataSet &dataSet);
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void LSMA_Channelv2::Calculate(DataSet &dataSet)
  {
   Data=GetPointer(dataSet);


// Reading the parameters
   BasePrice price=(BasePrice)ListParam[2].Index;
   int RPeriod=(int)NumParam[0].Value;
   double StDevOutside2=NumParam[1].Value;
   int iPrvs=CheckParam[0].Checked ? 1 : 0;

// Calculation
   double adPrice[]; Price(price,adPrice);
   double adUpBand[];  ArrayResize(adUpBand, Data.Bars); ArrayInitialize(adUpBand, 0);
   double adDnBand[];  ArrayResize(adDnBand, Data.Bars); ArrayInitialize(adDnBand, 0);
   double adCnBand[];  ArrayResize(adCnBand, Data.Bars); ArrayInitialize(adCnBand, 0);

   int iFirstBar=RPeriod+2;

   for(int iBar=iFirstBar; iBar<Data.Bars; iBar++)
     {
      double sum_xy= 0;
      double sum_x = 0;
      double sum_y = 0;
      double sum_x2= 0;
      double a = 0;
      double b = 0;
      double y1 = 0;
      double y2 = 0;
      double tmp_div=0;
      int n=RPeriod;

      double stddiv=0;
      double x2_1_upOut=0;
      double x2_1_downOut=0;

      int j = RPeriod;
      int i = RPeriod;

      //for (int x=0; x<RPeriod; x++) int i = iBar-length; i < iBar; i++, j--
      for(int x=iBar-RPeriod; x<iBar; x++,j--)
        {
         sum_xy= sum_xy+j*adPrice[iBar-j];
         sum_x = sum_x + j;
         sum_y = sum_y + adPrice[iBar-j];
         sum_x2= sum_x2+j*j;
        }
      // Calculate slope b and intercept a
      b = (n*sum_xy - sum_x*sum_y)/(n*sum_x2 - sum_x*sum_x);
      a = (sum_y - b*sum_x)/n;

      // Calculate endpoints for least squares line
      y1 = a + b*n;
      y2 = a + b;

      for(int x=iBar-RPeriod; x<iBar; x++,i--)
        {
         tmp_div=tmp_div+(adPrice[iBar-i]-(a+b*i))*(adPrice[iBar-i]-(a+b*i));
        }

      stddiv=MathSqrt(tmp_div/n);

      x2_1_upOut=y2+StDevOutside2*stddiv;
      x2_1_downOut=y2-StDevOutside2*stddiv;

      adUpBand[iBar] = x2_1_upOut;
      adDnBand[iBar] = x2_1_downOut;
      adCnBand[iBar]=tmp_div;//y2;
     }

// Saving the components

   ArrayResize(Component[0].Value,Data.Bars);
   Component[0].CompName = "Upper Band";
   Component[0].DataType = IndComponentType_IndicatorValue;
   Component[0].FirstBar = iFirstBar;
   ArrayCopy(Component[0].Value,adUpBand);

   ArrayResize(Component[1].Value,Data.Bars);
   Component[1].CompName = "Moving Average";
   Component[1].DataType = IndComponentType_IndicatorValue;
   Component[1].FirstBar = iFirstBar;
   ArrayCopy(Component[1].Value,adCnBand);

   ArrayResize(Component[2].Value,Data.Bars);
   Component[2].CompName = "Lower Band";
   Component[2].DataType = IndComponentType_IndicatorValue;
   Component[2].FirstBar = iFirstBar;
   ArrayCopy(Component[2].Value,adDnBand);

   ArrayResize(Component[3].Value,Data.Bars);
   Component[3].FirstBar=iFirstBar;

   ArrayResize(Component[4].Value,Data.Bars);
   Component[4].FirstBar=iFirstBar;

// Sets the Component's type.
   if(SlotType==SlotTypes_Open)
     {
      Component[3].DataType = IndComponentType_OpenLongPrice;
      Component[3].CompName = "Long position entry price";
      Component[4].DataType = IndComponentType_OpenShortPrice;
      Component[4].CompName = "Short position entry price";
     }
   else if(SlotType==SlotTypes_OpenFilter)
     {
      Component[3].DataType = IndComponentType_AllowOpenLong;
      Component[3].CompName = "Is long entry allowed";
      Component[4].DataType = IndComponentType_AllowOpenShort;
      Component[4].CompName = "Is short entry allowed";
     }
   else if(SlotType==SlotTypes_Close)
     {
      Component[3].DataType = IndComponentType_CloseLongPrice;
      Component[3].CompName = "Long position closing price";
      Component[4].DataType = IndComponentType_CloseShortPrice;
      Component[4].CompName = "Short position closing price";
     }
   else if(SlotType==SlotTypes_CloseFilter)
     {
      Component[3].DataType = IndComponentType_ForceCloseLong;
      Component[3].CompName = "Close out long position";
      Component[4].DataType = IndComponentType_ForceCloseShort;
      Component[4].CompName = "Close out short position";
     }

   if(SlotType==SlotTypes_Open || SlotType==SlotTypes_Close)
     {
      if(RPeriod>1)
        {
         for(int iBar=iFirstBar; iBar<Data.Bars; iBar++)
           { // Covers the cases when the price can pass through the band without a signal.
            double dOpen=Data.Open[iBar]; // Current open price

                                          // Upper band
            double dValueUp=adUpBand[iBar-iPrvs]; // Current value
            double dValueUp1=adUpBand[iBar-iPrvs-1]; // Previous value
            double dTempValUp=dValueUp;

            if((dValueUp1 > Data.High[iBar - 1]&& dValueUp < dOpen) || // The Data.Open price jumps above the indicator
               (dValueUp1 < Data.Low[iBar - 1] && dValueUp > dOpen) || // The Data.Open price jumps below the indicator
               (Data.Close[iBar-1]<dValueUp    && dValueUp < dOpen) || // The Data.Open price is in a positive gap
               (Data.Close[iBar-1]>dValueUp    &&  dValueUp> dOpen)) // The Data.Open price is in a negative gap
               dTempValUp=dOpen; // The entry/exit level is moved to Data.Open price

            // Lower band
            double dValueDown=adDnBand[iBar-iPrvs]; // Current value
            double dValueDown1=adDnBand[iBar-iPrvs-1]; // Previous value
            double dTempValDown=dValueDown;

            if((dValueDown1 > Data.High[iBar - 1]&& dValueDown<dOpen) || // The Data.Open price jumps above the indicator
               (dValueDown1 < Data.Low[iBar - 1] && dValueDown>dOpen) || // The Data.Open price jumps below the indicator
               (Data.Close[iBar-1]<dValueDown    && dValueDown<dOpen) || // The Data.Open price is in a positive gap
               (Data.Close[iBar-1]>dValueDown    &&  dValueDown>dOpen))  // The Data.Open price is in a negative gap
               dTempValDown=dOpen; // The entry/exit level is moved to Data.Open price

            if(ListParam[0].Text=="Enter long at the Upper Band" || 
               ListParam[0].Text=="Exit long at the Upper Band")
              {
               Component[3].Value[iBar] = dTempValUp;
               Component[4].Value[iBar] = dTempValDown;
              }
            else
              {
               Component[3].Value[iBar] = dTempValDown;
               Component[4].Value[iBar] = dTempValUp;
              }
           }
        }
      else
        {
         for(int iBar=2; iBar<Data.Bars; iBar++)
           {
            if(ListParam[0].Text=="Enter long at the Upper Band" || 
               ListParam[0].Text=="Exit long at the Upper Band")
              {
               Component[3].Value[iBar] = adUpBand[iBar - iPrvs];
               Component[4].Value[iBar] = adDnBand[iBar - iPrvs];
              }
            else
              {
               Component[3].Value[iBar] = adDnBand[iBar - iPrvs];
               Component[4].Value[iBar] = adUpBand[iBar - iPrvs];
              }
           }
        }
     }
   else
     {
      if(ListParam[0].Text=="The bar opens below the Upper Band")
        {
         BandIndicatorLogic(iFirstBar,iPrvs,adUpBand,adDnBand,Component[3],Component[4],
                            BandIndLogic_The_bar_opens_below_the_Upper_Band);
        }
      else if(ListParam[0].Text=="The bar opens above the Upper Band")
        {
         BandIndicatorLogic(iFirstBar,iPrvs,adUpBand,adDnBand,Component[3],Component[4],
                            BandIndLogic_The_bar_opens_above_the_Upper_Band);
        }
      else if(ListParam[0].Text=="The bar opens below the Lower Band")
        {
         BandIndicatorLogic(iFirstBar,iPrvs,adUpBand,adDnBand,Component[3],Component[4],
                            BandIndLogic_The_bar_opens_below_the_Lower_Band);
        }
      else if(ListParam[0].Text=="The bar opens above the Lower Band")
        {
         BandIndicatorLogic(iFirstBar,iPrvs,adUpBand,adDnBand,Component[3],Component[4],
                            BandIndLogic_The_bar_opens_above_the_Lower_Band);
        }
      else if(ListParam[0].Text=="The bar opens below the Upper Band after opening above it")
        {
         BandIndicatorLogic(iFirstBar,iPrvs,adUpBand,adDnBand,Component[3],Component[4],
                            BandIndLogic_The_bar_opens_below_Upper_Band_after_above);
        }
      else if(ListParam[0].Text=="The bar opens above the Upper Band after opening below it")
        {
         BandIndicatorLogic(iFirstBar,iPrvs,adUpBand,adDnBand,Component[3],Component[4],
                            BandIndLogic_The_bar_opens_above_Upper_Band_after_below);
        }
      else if(ListParam[0].Text=="The bar opens below the Lower Band after opening above it")
        {
         BandIndicatorLogic(iFirstBar,iPrvs,adUpBand,adDnBand,Component[3],Component[4],
                            BandIndLogic_The_bar_opens_below_Lower_Band_after_above);
        }
      else if(ListParam[0].Text=="The bar opens above the Lower Band after opening below it")
        {
         BandIndicatorLogic(iFirstBar,iPrvs,adUpBand,adDnBand,Component[3],Component[4],
                            BandIndLogic_The_bar_opens_above_Lower_Band_after_below);
        }
      else if(ListParam[0].Text=="The position opens above the Upper Band")
        {
         Component[0].PosPriceDependence = PositionPriceDependence_PriceBuyHigher;
         Component[2].PosPriceDependence = PositionPriceDependence_PriceSellLower;
         Component[0].UsePreviousBar = iPrvs;
         Component[2].UsePreviousBar = iPrvs;
         Component[3].DataType = IndComponentType_Other;
         Component[4].DataType = IndComponentType_Other;
         Component[3].ShowInDynInfo = false;
         Component[4].ShowInDynInfo = false;
        }
      else if(ListParam[0].Text=="The position opens below the Upper Band")
        {
         Component[0].PosPriceDependence = PositionPriceDependence_PriceBuyLower;
         Component[2].PosPriceDependence = PositionPriceDependence_PriceSellHigher;
         Component[0].UsePreviousBar = iPrvs;
         Component[2].UsePreviousBar = iPrvs;
         Component[3].DataType = IndComponentType_Other;
         Component[4].DataType = IndComponentType_Other;
         Component[3].ShowInDynInfo = false;
         Component[4].ShowInDynInfo = false;
        }
      else if(ListParam[0].Text=="The position opens above the Lower Band")
        {
         Component[0].PosPriceDependence = PositionPriceDependence_PriceSellLower;
         Component[2].PosPriceDependence = PositionPriceDependence_PriceBuyHigher;
         Component[0].UsePreviousBar = iPrvs;
         Component[2].UsePreviousBar = iPrvs;
         Component[3].DataType = IndComponentType_Other;
         Component[4].DataType = IndComponentType_Other;
         Component[3].ShowInDynInfo = false;
         Component[4].ShowInDynInfo = false;
        }
      else if(ListParam[0].Text=="The position opens below the Lower Band")
        {
         Component[0].PosPriceDependence = PositionPriceDependence_PriceSellHigher;
         Component[2].PosPriceDependence = PositionPriceDependence_PriceBuyLower;
         Component[0].UsePreviousBar = iPrvs;
         Component[2].UsePreviousBar = iPrvs;
         Component[3].DataType = IndComponentType_Other;
         Component[4].DataType = IndComponentType_Other;
         Component[3].ShowInDynInfo = false;
         Component[4].ShowInDynInfo = false;
        }
      else if(ListParam[0].Text=="The bar closes below the Upper Band")
        {
         BandIndicatorLogic(iFirstBar,iPrvs,adUpBand,adDnBand,Component[3],Component[4],
                            BandIndLogic_The_bar_closes_below_the_Upper_Band);
        }
      else if(ListParam[0].Text=="The bar closes above the Upper Band")
        {
         BandIndicatorLogic(iFirstBar,iPrvs,adUpBand,adDnBand,Component[3],Component[4],
                            BandIndLogic_The_bar_closes_above_the_Upper_Band);
        }
      else if(ListParam[0].Text=="The bar closes below the Lower Band")
        {
         BandIndicatorLogic(iFirstBar,iPrvs,adUpBand,adDnBand,Component[3],Component[4],
                            BandIndLogic_The_bar_closes_below_the_Lower_Band);
        }
      else if(ListParam[0].Text=="The bar closes above the Lower Band")
        {
         BandIndicatorLogic(iFirstBar,iPrvs,adUpBand,adDnBand,Component[3],Component[4],
                            BandIndLogic_The_bar_closes_above_the_Lower_Band);
        }
     }

  }
//+------------------------------------------------------------------+
